<?php

namespace Teknisa\SUP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use \Teknisa\Libs\Util\Utilities as LibUtilities;

class IntegracaoMercadumController
{

  	protected $connection;
  	protected $environment;
  	protected $utilities;
  	
    public function __construct($connection, $environment, LibUtilities $utilities){
        $this->connection  = $connection;
        $this->environment = $environment;
        $this->utilities   = $utilities;
    }
    
    public function enviaDadosIntegracao ($dados, $url){
        try{
            $postdata = http_build_query($dados);
            $ch = curl_init();
            curl_setopt($ch,CURLOPT_URL, $url);
            curl_setopt($ch,CURLOPT_POST, true);
            curl_setopt($ch,CURLOPT_POSTFIELDS, $postdata);
            curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
            $result = curl_exec($ch);
        } catch (Exception $e) {
            $error = new DTO\Response\Error($e->getMessage(), $e->getCode());
              $response->setError($error);
        }
    }

    private function getUserData(){
        $cdoperador = $this->environment->getCdOperador();
        $sql = "SELECT NRORGOPER, NRORG AS ORGANIZATION_ID FROM OPERADOR WHERE CDOPERADOR = :P_CDOPERADOR";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            'P_CDOPERADOR' => $cdoperador
        ));
        return $stmt->fetch();
    }

    protected function readOAccCustom() {
        $oAccCustom = array();
        $occCustomPath = InstanceManager::getInstance()->getParameter('oAccCustom');
        $modulesPath = __DIR__ . '/../../../../../..';
        $loginPath = $modulesPath . '/login/backend/config/';   
        $fileDir = $loginPath . $occCustomPath;
        try {
            if (file_exists($fileDir)) {
                $fileContents  = file_get_contents($fileDir);
                $fileDecrypted = LibUtilities::zeedhiDecrypt($fileContents, LibUtilities::LOGIN_CRYPT_STRING);
                $fileDecoded   = @json_decode($fileDecrypted, true);
                if(isset($fileDecoded['organizationId']) && 
                    isset($fileDecoded['dbHost']) && 
                    is_numeric($fileDecoded['organizationId'])) {
                    $oAccCustom = $fileDecoded;
                }
            }
        } catch(\Exception $e) {}

        return $oAccCustom;
    }

    private function getTeknisaOrganizationId() {
        $userData       = $this->getUserData();
        $oAccCustom     = $this->readOAccCustom();
        $nrOrgOper      = $userData['NRORGOPER'];
        $organizationId = $userData['ORGANIZATION_ID'];
        
        if(!empty($nrOrgOper)) {
            return $nrOrgOper;
        }
        if(!empty($oAccCustom)) {
            return $oAccCustom['organizationId'];
        }
        return $organizationId;
    }

    // Supplier
    public function integracaoMercadumSupplier ($row){
        try{
            $fornecedores = $this->buscaFornecedores();
            $mercadumIntegration = $this->buildFormattedData_Supplier($row,$fornecedores);
            //$url = 'http://paulooliveira1.zeedhi.com/workfolder/mercadum/mercadum-fornecedor/backend/api/integration/suppliers';
            //$return = $this->enviaDadosIntegracao($mercadumIntegration, $url);
        } catch (Exception $e) {
            $error = new DTO\Response\Error($e->getMessage(), $e->getCode());
		    $response->setError($error);
        }
    }
    
    public function buscaFornecedores (){
        try {
            $query = "  SELECT  DF.NMUSUARIOECO, DF.CDOPERFORN, DF.DSLOGINECO, F.CDFORNECED, F.IDTPIJURFORN,
                                F.NRINSJURFORN, F.NMRAZSOCFORN, F.NMFANTFORN, F.NRINSESTFORN, E.IDTPENDEFORN, F.VRMINAUTFOR, F.VRPERVARDESCPREC, F.VRPERVARPREC,
                                E.CDPAIS, E.SGESTADO, E.CDMUNICIPIO, E.DSENDEFORN, E.NMBAIRFORN, E.NRCEPFORN
                        FROM FORNECEDOR F 
                                INNER JOIN ENDEFORN E ON F.CDFORNECED = E.CDFORNECED
                                INNER JOIN DADOSFORNECOMMERCE DF ON F.CDFORNECED = DF.CDFORNECED
                        WHERE E.IDTPENDEFORN = :P_IDTPENDEFORN";
                    
            $params = array(
                "P_IDTPENDEFORN" => "P"
            );        
            
            $std = $this->connection->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function buildFormattedData_Supplier($row, $fornecedores){
		$mercadumIntegration = array();
		$dados = array();		
		if($row['NRORG'] == "1"){
		    $dados['Nrorg'] = $this->getTeknisaOrganizationId();
		} else {
		    $dados['Nrorg'] = $row['NRORG'];
		}
		$dados['Chave'] = $row['CHAVE'];
		
		if (count($fornecedores)) {
			$fornecedoresList = array();
			foreach ($fornecedores as $fornecedor) {
			    $dadosFornecedor = array();
				$dadosFornecedor['SUPPLIER']         =  $this->buildSupplier($fornecedor);
				$dadosFornecedor['SUPPLIER_ADDRESS'] =  $this->buildSupplierAddress($fornecedor);
				$dadosFornecedor['USERS']            =  $this->buildUsers($fornecedor);
				array_push($fornecedoresList, $dadosFornecedor);
			}
    	    $dados['Fornecedores'] = $fornecedoresList;
    	    $mercadumIntegration["mercadumIntegration"] = $dados;
			return $mercadumIntegration;
		} else {
			throw new \Exception("Não foram encontrados Fornecedores.");
		}
    }
    
    public function buildSupplier($fornecedor){
		$supplier = array(
			"SUPPLIER_ID"        => $fornecedor['CDFORNECED'],
			"TYPE_SUPPLIER"      => $fornecedor['IDTPIJURFORN'],
			"REGISTRATION"       => $fornecedor['NRINSJURFORN'],
			"CORPORATE_NAME"     => $fornecedor['NMRAZSOCFORN'],
			"SUPPLIER_NAME"      => $fornecedor['NMFANTFORN'],
			"STATE_REGISTRATION" => $fornecedor['NRINSESTFORN'],
			"MINIMUM_BILLING"    => $fornecedor['VRMINAUTFOR'],
			"DISCOUNT_VALUE"     => $fornecedor['VRPERVARDESCPREC'],
			"ADDITION_VALUE"     => $fornecedor['VRPERVARPREC']
		);
		return $supplier;
    }
     
    public function buildSupplierAddress($fornecedor){
		$supplierAddress = array(
			"SUPPLIER_ADDRESS_ID" => $fornecedor['IDTPENDEFORN'],
			"COUNTRY_CODE"        => $fornecedor['CDPAIS'],
			"STATE_CODE"          => $fornecedor['SGESTADO'],
			"CITY_CODE"           => $fornecedor['CDMUNICIPIO'],
			"ADDRESS"             => $fornecedor['DSENDEFORN'],
			"DISTRICT"            => $fornecedor['NMBAIRFORN'],
			"ZIP_CODE"            => $fornecedor['NRCEPFORN']
		); 
		return $supplierAddress;
    }
    
    public function buildUsers($fornecedor){
		$users = array(
			"USER_NAME"  => $fornecedor['NMUSUARIOECO'],
			"USER_ID"    => $fornecedor['CDOPERFORN'],
			"EMAIL"      => $fornecedor['DSLOGINECO']
		);
		return $users;
    }
    //

    
    // Pedido de Coleta
    public function integracaoMercadumPedidoColeta ($row){
        try{
            /*
            $buscaPedidoColeta = $this->buscaPedidoColeta();
            $mercadumIntegration = $this->buildFormattedData_PedidoColeta($row,$buscaPedidoColeta);
            //$url = '';
            //$return = $this->enviaDadosIntegracao($mercadumIntegration, $url);
            */
        } catch (Exception $e) {
            $error = new DTO\Response\Error($e->getMessage(), $e->getCode());
		    $response->setError($error);
        }
    }
    
    public function buildFormattedData_PedidoColeta($row, $pedidos){
		$mercadumIntegration = array();
		$dados = array();
		$dados['Nrorg'] = $row['NRORG'];
		$dados['Chave'] = $row['CHAVE'];
		
		if (count($pedidos)) {
			$PedidosList = array();
			foreach ($pedidos as $pedido) {
			    $dadosPedido = array();
				$dadosPedido['SUPPLIER']         =  $this->buildPedidoColeta_Supplier($pedido);
				array_push($PedidosList, $dadosPedido);
			}
    	    $dados['PedidosColeta'] = $PedidosList;
    	    $mercadumIntegration["mercadumIntegration"] = $dados;
			return $mercadumIntegration;
		} else {
			throw new \Exception("Não foram encontrados Pedidos de Coleta.");
		}
    }
    
    public function buildPedidoColeta_Supplier($pedido){
		$supplier = array(
			"SUPPLIER_ID"        => $pedido['CDFORNECED'],
			"CORPORATE_NAME"     => $pedido['NMRAZSOCFORN'],
			"REGISTRATION"       => $pedido['NRINSJURFORN'],
			"TYPE_SUPPLIER"      => $pedido['IDTPIJURFORN']
		);
		return $supplier;
    }
    
    public function buildPedidoColeta_Prefab($pedido){
		$array = array(
			""       => $pedido['NRPEDIDOCOL'],
			""       => $pedido['VRFRETEFORNEPEDCOL'],
			""       => $pedido['IDPEDCOLCONS'],
			""       => $pedido['DTPEDCOLETA'],
			""       => $pedido['IDSITUPEDCOLETA'],
			""       => $pedido['ST_DESCRIPTION'],
			""       => $pedido['CDFILIAL'],
			""       => $pedido['CDREGIAO'],
			""       => $pedido['DATA'],
			""       => $pedido['DTINIPERGERAL'],
			""       => $pedido['DTFINPERGERAL'],
			""       => $pedido['VALIDADE'],
			""       => $pedido['COD_COMPRADOR'],
			""       => $pedido['COD_RESPONSAVEL'],
			""       => $pedido['NM_COMPRADOR'],
			""       => $pedido['NM_RESPONSAVEL'],
			""       => $pedido['EMAIL'],
			""       => $pedido['DTEXPIRAPEDCOL'],
			""       => $pedido['PRODCOT'],
			""       => $pedido['IDEXIBEQTDCMP'],
			""       => $pedido['IDRENEGOCIAPCOL'],
			""       => $pedido['NRORG']
		);
		return $array;
    }
    
    public function buscaPedidoColeta ($row){
        try {
            $query = "  SELECT DISTINCT 
                            FO.CDFORNECED,
                            PE.NRPEDIDOCOL,
                            FO.NMRAZSOCFORN,
                            FO.NRINSJURFORN,
                            PD.VRFRETEFORNEPEDCOL,
                            FO.IDTPIJURFORN,
                            PE.IDPEDCOLCONS,
                            TO_CHAR(PE.DTPEDCOLGERAL,'DD/MM/YYYY') AS DTPEDCOLETA,
                            PE.IDSITUPEDCOLETA,
                            CASE IDSITUPEDCOLETA
                                WHEN 'N'
                                THEN 'PENDENTE'
                                WHEN 'A'
                                THEN 'ACEITA'
                                WHEN 'R'
                                THEN 'RECUSADA'
                                WHEN 'T'
                                THEN 'AGUARDANDO ACEITE FORNECEDOR'
                                WHEN 'C'
                                THEN 'AGUARDANDO ACEITE COMPRADOR'
                                WHEN 'I'
                                THEN 'INCONSISTÊNCIAS DO PEDIDO'
                            END AS ST_DESCRIPTION,
                            PE.CDFILIAL,
                            FI.CDREGIAO,
                            PE.DTPEDCOLGERAL DATA,
                            PE.DTINIPERGERAL,
                            PE.DTFINPERGERAL,
                            ( 
                                TO_CHAR(PE.DTINIPERGERAL,'DD/MM/YYYY')
                                || ' ATÉ '
                                || TO_CHAR(PE.DTFINPERGERAL,'DD/MM/YYYY') 
                            ) VALIDADE,
                            COMP.CDOPERADOR COD_COMPRADOR,
                            PE.CDOPERLANC COD_RESPONSAVEL,
                            COMP.NMOPERADOR NM_COMPRADOR,
                            RESP.NMOPERADOR NM_RESPONSAVEL,
                            RESP.DSEMAILOPER EMAIL,
                            TO_CHAR(PE.DTEXPIRAPEDCOL,'DD/MM/YYYY HH24:MI:SS') AS DTEXPIRAPEDCOL,
                            (
                                SELECT COUNT(*) PRODCOTADO
                                FROM PRMARCPEDCOL
                                WHERE CDFILIAL       = PE.CDFILIAL
                                AND NRPEDIDOCOL      = PE.NRPEDIDOCOL
                                AND VRPRECUNIPRMARC IS NOT NULL
                                AND NRORG            = :NRORG
                            )
                            || '/'
                            ||
                            (
                                SELECT COUNT(PRM.CDPRODUTO) TOTALPROD
                                FROM 
                                    PRMARCPEDCOL PRM,
                                    PRODUTO PR
                                WHERE 
                                    PRM.CDPRODUTO = PR.CDPRODUTO
                                    AND UPPER(PR.NMPRODUTO) NOT LIKE '%(INATIVO)%'
                                    AND PRM.CDFILIAL    = PE.CDFILIAL
                                    AND PRM.NRPEDIDOCOL = PE.NRPEDIDOCOL
                                    AND PRM.NRORG       = :NRORG
                            ) PRODCOT,
                            NVL(PARAM.IDEXIBEQTDCMP,'N') IDEXIBEQTDCMP,
                            PE.IDRENEGOCIAPCOL,
                            PE.NRORG
                        FROM    PEDCOLGERAL PE,
                                PEDCOLETA PD,
                                FORNECEDOR FO,
                                OPERADOR COMP,
                                OPERADOR RESP,
                                FILIAL FI,
                                PARAMGERALECOMMERCE PARAM
                        WHERE   
                            FO.CDFORNECED = :P_CDFORNECED
                            AND PE.CDFILIAL     = FI.CDFILIAL
                            AND PE.NRORG        = FI.NRORG
                            AND PE.CDOPERADOR   = COMP.CDOPERADOR
                            AND PE.NRPEDIDOCOL  = PD.NRPEDIDOCOL
                            AND PE.NRORG        = COMP.NRORG
                            AND PE.CDOPERLANC   = RESP.CDOPERADOR
                            AND PE.NRORG        = RESP.NRORG
                            AND PARAM.CDFILIAL  = PE.CDFILIAL
                            AND PARAM.NRORG     = PE.NRORG
                            AND ( ( TRUNC(PE.DTEXPIRAPEDCOL) BETWEEN TO_DATE(:P_DTINICIAL,'DD/MM/YYYY') AND TO_DATE(:P_DTFINAL,'DD/MM/YYYY') )
                            OR (:P_TODOS                     = 'T' ) )
                            AND ( ( TRUNC(PE.DTINIPERGERAL) >= TO_DATE(:P_DTVALINI,'DD/MM/YYYY')
                            AND TRUNC(PE.DTFINPERGERAL)     <= TO_DATE(:P_DTVALFIN,'DD/MM/YYYY') )
                            OR (:P_TDTVALIDADE               = 'T' ) )
                            AND ( (:P_TEXPIRADAS             = 'T'
                            AND PE.DTEXPIRAPEDCOL            > SYSDATE )
                            OR ( (:P_TEXPIRADAS             IS NULL )
                            AND PE.DTEXPIRAPEDCOL            < SYSDATE ) )
                            AND EXISTS
                            (
                                SELECT 1
                                FROM PEDCOLETA PD
                                WHERE   PD.CDFILIAL  = PE.CDFILIAL
                                    AND PD.NRPEDIDOCOL = PE.NRPEDIDOCOL
                                    AND PD.NRORG       = PE.NRORG
                                    AND PD.CDFORNECED  = FO.CDFORNECED
                                    AND PD.NRORG       = FO.NRORG
                                    AND PD.NRORG       =:NRORG
                            )
                            AND ( INSTR(NVL(:P_STPEDCOLETA,PE.IDSITUPEDCOLETA),PE.IDSITUPEDCOLETA) > 0 )
                            AND PE.NRORG                                                           =:NRORG
                        ORDER BY    
                            DECODE(PE.IDSITUPEDCOLETA,'N',0,10),
                            PE.NRPEDIDOCOL DESC";
                    
            $params = array(
                "NROG"          => $row["NRORG"],
                "P_CDFORNECED"  => $row["FORNECED"],
                "P_DTINICIAL"   => $row["DTINI"],
                "P_DTFINAL"     => $row["DTFIN"],
                "P_TODOS"       => $row["TODOS"], //?
                "P_DTVALINI"    => $row["DTVALINI"], //?
                "P_DTVALFIN"    => $row["DTVALFIN"], //?
                "P_TDTVALIDADE" => $row["DTEXPIRAPEDCOL"],
                "P_TEXPIRADAS"  => $row["TEXPIRADAS"], //?
                "P_STPEDCOLETA" => $row["STPEDCOLETA"] //?? 
            );        
            
            $std = $this->connection->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    //
}
