<?php
namespace Teknisa\SUP\Modules\Cotacao\Modules\ListaPreco\Controllers;

use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\SUP\Util\SupUtilities;
use Zeedhi\Framework\DataSource\DataSet;

class SupCustoListaInflacionadaController extends \Zeedhi\Framework\Controller\Report {
    
    protected $entityManager;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    protected $supUtilities;
    
    public function __construct($entityManager, $newCode, $environment, $connection, $dataSourceManager, SupUtilities $supUtilities) {
        
        $this->entityManager     = $entityManager;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->supUtilities  = $supUtilities;
    }
    
    //caso no storage n tenha o CDFILCENTCMP no Local Store
    public function validacoesIniciais(DTO\Request\Row $request, DTO\Response $response)
    {
        try{
            $row = $request->getRow();
            $centralCompras = $this->supUtilities->getCentralCompras();
            $response->addDataSet(new DataSet("centralCompras", $centralCompras));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    //------------------------Atualizar Custo-----------------------///
    public function atualizarCusto(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        $connection = $this->connection;
        $procedure = new StoredProcedure($connection, 'CUSTO_LISTAINFLAC');
        $procedure->addParam(new Param('P_CDFILCENTCMP', Param::PARAM_INPUT, $row['P_CDFILCENTCMP'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLISTINFLAC', Param::PARAM_INPUT, $row['P_NRLISTINFLAC'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $row['P_CDOPERADOR'], Param::PARAM_TYPE_STR));
        
        
        $procedure->execute();
    }
    

    //------------------------------ PRODUTO --------------------------------//
    
    /* ****************
     *    DELETE      *
     ******************/
     
    public function excluirProduto(DTO\Request $request, DTO\Response $response) {
        
        $row = $request->getRow(); 
        
        $params = array(
		    "P_CDFILCENTCMP" => $row["P_CDFILCENTCMP"],
		    "P_NRLISTINFLAC" => $row["P_NRLISTINFLAC"],
		    "P_CDPRODUTO" => $row["P_CDPRODUTO"],
		);
        
        
        $query = 'DELETE FROM PERLISINFLAC
                    WHERE
                        CDFILCENTCMP = :P_CDFILCENTCMP AND
                        NRLISTINFLAC = :P_NRLISTINFLAC AND
                        CDPRODUTO = :P_CDPRODUTO';
		
		
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
    }
    
    
    /* ***************
     *    INSERT     *
     *****************/
   public function insertProduto(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow(); 
        
        $params = array(
		    "P_CDFILCENTCMP" => $row["P_CDFILCENTCMP"],
    	    "P_NRLISTINFLAC" => $row["P_NRLISTINFLAC"],
    	    "P_CDPRODUTO" => $row["P_CDPRODUTO"],
    	    "P_VRPERLISTINF" => $row["P_VRPERLISTINF"]
		);

        $query = 'INSERT INTO PERLISINFLAC
                    (CDFILCENTCMP, NRLISTINFLAC, CDPRODUTO, VRPERLISTINF)
                  VALUES
                    (:P_CDFILCENTCMP, :P_NRLISTINFLAC, :P_CDPRODUTO, :P_VRPERLISTINF)';
	    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
        
    }
    /* ****************
     *    UPDATE      *
     ******************/
  public function updateProduto(DTO\Request $request, DTO\Response $response) {
        
        $row = $request->getRow();
    
        $floatVR = floatval($row['P_VRPERLISTINF']);
            
    	$params = array(
		    "P_VRPERLISTINF" => $floatVR,
		    "P_CDFILCENTCMP" => $row["P_CDFILCENTCMP"],
		    "P_NRLISTINFLAC" => $row["P_NRLISTINFLAC"],
		    "P_CDPRODUTO" => $row["P_CDPRODUTO"]
		    
		);
        
        $query = 'UPDATE PERLISINFLAC 
                        SET VRPERLISTINF = :P_VRPERLISTINF 
                    WHERE 
                        CDFILCENTCMP = :P_CDFILCENTCMP AND 
                        NRLISTINFLAC = :P_NRLISTINFLAC AND 
                        CDPRODUTO = :P_CDPRODUTO';
		$stmt = $this->connection->prepare($query);
	
		$stmt->execute($params);
    }
    
    //------------------------LISTA INFLACIONADA-----------------------///
    
    
    /* ***************
     *    INSERT     *
     *****************/
    
   public function insertLista(DTO\Request $request, DTO\Response $response) {
       
       $row = $request->getRow();
       
       $nrlista = $this->newCode->getCodeByTableName('LISTAINFLAC', $this->environment->getNrorgTrab());
       
       
       $row['NRLISTINFLAC'] = $nrlista;
      
       $params = array(
                "P_CDFILCENTCMP" => $row["P_CDFILCENTCMP"],
                "P_NRLISTINFLAC" => $row["NRLISTINFLAC"],
                "P_NMLISTINFLAC" => $row["P_NMLISTINFLAC"],
                "P_DTREFLISTINF" => $row["P_DTREFLISTINF"],
                "P_DTINILISTINF" => $row["P_DTINILISTINF"],
                "P_DTFINLISTINF" => $row["P_DTFINLISTINF"],
                "P_CDOPERADOR" => $row["P_CDOPERADOR"],
                "P_IDLISINFATIV" => 'S'
        );
       
		$query ='INSERT INTO LISTAINFLAC
                    (CDFILCENTCMP, NRLISTINFLAC, NMLISTINFLAC, DTREFLISTINF,
                    DTINILISTINF, 
                    DTFINLISTINF, CDOPERADOR, DTGERLISTINF, DTULTATULISI, IDLISINFATIV)
                VALUES
                    (:P_CDFILCENTCMP, :P_NRLISTINFLAC, :P_NMLISTINFLAC, :P_DTREFLISTINF,
                    :P_DTINILISTINF, 
                    :P_DTFINLISTINF, :P_CDOPERADOR, SYSDATE, SYSDATE,
                    :P_IDLISINFATIV)';
        $stmt = $this->connection->prepare($query);
        
        $stmt->execute($params);
                    
	}
	
	/* ****************
     *    DELETE      *
     ******************/
	
    
    public function deletListaInflacionada(DTO\Request $request, DTO\Response $response) {
      
        $row = $request->getRow();
        $params = array(
            "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
            "P_NRLISTINFLAC" => $row['P_NRLISTINFLAC']
        );
        
      
      try{
            $this->connection->beginTransaction();
    
            //-------------FIRST DELET-------------------
            
            $query2 = "DELETE FROM PERLISINFLAC
                        WHERE
                            CDFILCENTCMP = :P_CDFILCENTCMP AND
                            NRLISTINFLAC = :P_NRLISTINFLAC";
            $stmt = $this->connection->prepare($query2);
            $stmt->execute($params);
            $this->connection->commit();
            
          } catch (Exception $e){
              $this->connection->rollback();
              throw $e;
        }
        
        try{
            $this->connection->beginTransaction();
            
            //-------------SECOND DELET-------------------
    
        
    		$query = "DELETE FROM LISTAINFLAC
                        WHERE
                            CDFILCENTCMP = :P_CDFILCENTCMP AND
                            NRLISTINFLAC = :P_NRLISTINFLAC";
                            
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
    		
            $this->connection->commit();
      } catch (Exception $e){
          $this->connection->rollback();
          throw $e;
      }
        
	}
	
	/* ****************
     *    UPDATE      *
     ******************/
    public function updateListaInflacionada(DTO\Request\Row $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        
        $param = array(
            "P_NMLISTINFLAC" => $row['NMLISTINFLAC'],
            "P_DTREFLISTINF" => $row['DTREFLISTINF'],
            "P_DTINILISTINF" => $row['DTINILISTINF'],
            "P_DTFINLISTINF" => $row['DTFINLISTINF'],
            "P_IDLISINFATIV" => $row['IDLISINFATIV'],
            "P_CDFILCENTCMP" => $row['CDFILCENTCMP'],
            "P_NRLISTINFLAC" => $row['NRLISTINFLAC']
        );
       
        $query = "UPDATE LISTAINFLAC
                    SET
                      NMLISTINFLAC = :P_NMLISTINFLAC,
                      DTREFLISTINF = :P_DTREFLISTINF,
                      DTINILISTINF = :P_DTINILISTINF,
                      DTFINLISTINF = :P_DTFINLISTINF,
                      IDLISINFATIV = :P_IDLISINFATIV
                    WHERE
                      CDFILCENTCMP = :P_CDFILCENTCMP AND
                      NRLISTINFLAC = :P_NRLISTINFLAC";
                      
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($param);
        
    }
    
}