<?php
namespace Teknisa\SUP\Modules\Fornecimento\Modules\Relatorios\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\SUP\Modules\Fornecimento\Modules\Relatorios\Services\Sup01900ProdutosCotadosService;

class Sup01900ProdutosCotadosController extends \Zeedhi\Framework\Controller\Report
{
    protected $qr2Strategy;
    protected $environment;
    
    public function __construct($reportService, $qr2Strategy, $environment, Sup01900ProdutosCotadosService $service)
    {
        $this->service           = $service;
        $this->qr2Strategy       = $qr2Strategy;
        $this->reportService     = $reportService;
        $this->environment       = $environment;
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $row['__report_name'] = $row['REPORT'];
            
            $params = array(
                "CDFILIAL" => $this->environment->getCdFilial()
            );
            
            $row['CDFILIAL'] = $this->service->getFilial($params);
            
            $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        } catch (Exception $e) {
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        }
    
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] =
            array (
                'strategy' => $row['STRATEGY'],
                'parameterMapping' =>
                    array (
                        "CDFILIAL"      => "P_CDFILIAL",
                        "DTINI"         => "P_DTINI",
                        "DTFIN"         => "P_DTFIN",
                        "DTINICIAL"     => "P_DTINICIAL",
                        "DTFINAL"       => "P_DTFINAL",
                        "CDARVPRODINI"  => "P_CDINI",
                        "CDARVPRODFIN"  => "P_CDFIN",
                        "CDFORNECED"    => "L_CDFORNECED",
                        "CDREGIAO"      => "L_CDREGIAO"
                    ),
            );
     }
}
